import json
import os
from alpaka.alpaka import Alpaka
from alpaka.ollama_client import OllamaClient
from alpaka.rath import AlpakaRath, AlpakaLinkComposition
from rath.links.split import SplitLink
from fakts_next.contrib.rath.aiohttp import FaktsAIOHttpLink
from fakts_next.contrib.rath.graphql_ws import FaktsGraphQLWSLink
from herre_next.contrib.rath.auth_link import HerreAuthLink
from graphql import OperationType
from herre_next import Herre
from fakts_next import Fakts

from arkitekt_next.base_models import Manifest


from arkitekt_next.service_registry import (
    BaseArkitektService,
    Params,
    get_default_service_registry,
)
from arkitekt_next.base_models import Requirement
from alpaka.vars import current_alpaka


class ArkitektOllamaClient(OllamaClient):
    fakts: Fakts

    async def aconnect(self, **kwargs):
        self.endpoint_url = await self.fakts.aget("ollama.endpoint_url")
        return await super().aconnect(**kwargs)

class ArkitektNextAlpaka(Alpaka):
    rath: AlpakaRath
    ollama: ArkitektOllamaClient


def build_relative_path(*path: str) -> str:
    return os.path.join(os.path.dirname(__file__), *path)


class AlpakaService(BaseArkitektService):

    def get_service_name(self):
        return "alpaka"

    def build_service(
        self, fakts: Fakts, herre: Herre, params: Params, manifest: Manifest
    ):
        return ArkitektNextAlpaka(
            rath=AlpakaRath(
                link=AlpakaLinkComposition(
                    auth=HerreAuthLink(herre=herre),
                    split=SplitLink(
                        left=FaktsAIOHttpLink(
                            fakts_group="alpaka", fakts=fakts, endpoint_url="FAKE_URL"
                        ),
                        right=FaktsGraphQLWSLink(
                            fakts_group="alpaka", fakts=fakts, ws_endpoint_url="FAKE_URL"
                        ),
                        split=lambda o: o.node.operation != OperationType.SUBSCRIPTION,
                    ),
                )
            ),
            ollama=ArkitektOllamaClient(fakts=fakts, endpoint_url="FAKE_URL"),
        )

    def get_requirements(self):
        return [
            Requirement(
                key="alpaka",
                service="live.arkitekt.alpaka",
                description="An instance of ArkitektNext alpaka to retrieve graphs from",
            ),
            Requirement(
                key="ollama",
                service="io.ollama.ollama",
                description="An instance of Ollama to chat with",
            )
        ]

    def get_graphql_schema(self):
        schema_graphql_path = build_relative_path("api", "schema.graphql")
        with open(schema_graphql_path) as f:
            return f.read()

    def get_turms_project(self):
        turms_prject = build_relative_path("api", "project.json")
        with open(turms_prject) as f:
            return json.loads(f.read())





get_default_service_registry().register(AlpakaService())
